; GIMP - The GNU Image Manipulation Program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
;
; 
; Create an animated like "flowing sand" or a thick, viscous, grainy liquid material.  
;
; The script can be found on an open image in GIMP 2.4 under: Filters > Animation > Flowing Sand...
; Under GIMP 2.2, it can be found under: Filters > Animation > Animators > Flowing Sand...

; The animation may be saved with the gif-plug-in.
;
; The script lets you choose the image dimensions, number of frames in the animation
; the speed of the animation, the blend mode of the upper layer, whether or not
; you add the noise interactively (creates the lava) or let the script automatically do it
; for you and the lava color.

; Define the Function

(define (script-fu-flowing-sand-anim 	img		 
				drawable
				inWidth
				inHeight
				counter
				inSpeed
				upperBlendOpt
				middleBlendOpt
				interactive
				emboss
				color
				colorBlendOpt
				)

; Declare the Variables

	(let* (
		    	(newImage 0)		    
			(width (+ 10 inWidth))
			(height (+ 10 inHeight))			
			(bottomSandLayer 0)
			(bottomSandLayerCopy 0)
			(middleSandLayer 0)
			(middleSandLayerCopy 0)
			(topSandLayer 0)
			(topSandLayerCopy 0)
			(upperBlendMode
				(cond 
				(( equal? upperBlendOpt 0 ) HARDLIGHT-MODE)
				(( equal? upperBlendOpt 1 ) DIVIDE-MODE)
				(( equal? upperBlendOpt 2 ) SCREEN-MODE)
				(( equal? upperBlendOpt 3 ) OVERLAY-MODE)
				(( equal? upperBlendOpt 4 ) DODGE-MODE)
				(( equal? upperBlendOpt 5 ) BURN-MODE)
				(( equal? upperBlendOpt 6 ) MULTIPLY-MODE)
				(( equal? upperBlendOpt 7 ) SOFTLIGHT-MODE)
				(( equal? upperBlendOpt 8 ) GRAIN-EXTRACT-MODE)
				(( equal? upperBlendOpt 9 ) GRAIN-MERGE-MODE)
				(( equal? upperBlendOpt 10 ) DIFFERENCE-MODE)
				(( equal? upperBlendOpt 11 ) ADDITION-MODE)
				(( equal? upperBlendOpt 12 ) SUBTRACT-MODE)
				(( equal? upperBlendOpt 13 ) DARKEN-ONLY-MODE)
				(( equal? upperBlendOpt 14 ) LIGHTEN-ONLY-MODE)
				)	
			)
			(middleBlendMode
				(cond 
				(( equal? middleBlendOpt 0 ) SOFTLIGHT-MODE)
				(( equal? middleBlendOpt 1 ) DIVIDE-MODE)
				(( equal? middleBlendOpt 2 ) SCREEN-MODE)
				(( equal? middleBlendOpt 3 ) OVERLAY-MODE)
				(( equal? middleBlendOpt 4 ) DODGE-MODE)
				(( equal? middleBlendOpt 5 ) BURN-MODE)
				(( equal? middleBlendOpt 6 ) HARDLIGHT-MODE)
				(( equal? middleBlendOpt 7 ) MULTIPLY-MODE)
				(( equal? middleBlendOpt 8 ) GRAIN-EXTRACT-MODE)
				(( equal? middleBlendOpt 9 ) GRAIN-MERGE-MODE)
				(( equal? middleBlendOpt 10 ) DIFFERENCE-MODE)
				(( equal? middleBlendOpt 11 ) ADDITION-MODE)
				(( equal? middleBlendOpt 12 ) SUBTRACT-MODE)
				(( equal? middleBlendOpt 13 ) DARKEN-ONLY-MODE)
				(( equal? middleBlendOpt 14 ) LIGHTEN-ONLY-MODE)
				)	
			)			
			(colorBlend
				(cond 
				(( equal? colorBlendOpt 0 ) OVERLAY-MODE)
				(( equal? colorBlendOpt 1 ) DIVIDE-MODE)
				(( equal? colorBlendOpt 2 ) SCREEN-MODE)
				(( equal? colorBlendOpt 3 ) DODGE-MODE)
				(( equal? colorBlendOpt 4 ) BURN-MODE)
				(( equal? colorBlendOpt 5 ) MULTIPLY-MODE)
				(( equal? colorBlendOpt 6 ) HARDLIGHT-MODE)
				(( equal? colorBlendOpt 7 ) SOFTLIGHT-MODE)
				(( equal? colorBlendOpt 8 ) GRAIN-EXTRACT-MODE)
				(( equal? colorBlendOpt 9 ) GRAIN-MERGE-MODE)
				(( equal? colorBlendOpt 10 ) DIFFERENCE-MODE)
				(( equal? colorBlendOpt 11 ) ADDITION-MODE)
				(( equal? colorBlendOpt 12 ) SUBTRACT-MODE)
				(( equal? colorBlendOpt 13 ) DARKEN-ONLY-MODE)
				(( equal? colorBlendOpt 14 ) LIGHTEN-ONLY-MODE)
				(( equal? colorBlendOpt 15 ) COLOR-MODE)
				)	
			)			
			(layerName 0)
			(runMode 0)
			(offset 0)
			(offsetBottom 0)
			(step 0)
			(stepBottom 0)
			(frameNum 1)
	            (colorLayer 0)
		)

(gimp-context-push)

(set! newImage (car (gimp-image-new width height RGB)))
(gimp-image-undo-disable newImage)
(set! bottomSandLayer (car (gimp-layer-new newImage width height RGBA-IMAGE "Bottom Sand Layer" 100 NORMAL-MODE)))
(gimp-image-add-layer newImage bottomSandLayer -1)

(if (= interactive TRUE)
	(set! runMode RUN-INTERACTIVE)
	(set! runMode RUN-NONINTERACTIVE)
)

(plug-in-solid-noise runMode newImage bottomSandLayer 1 1 (rand) 1 2.0 2.0)
(set! middleSandLayer (car (gimp-layer-copy bottomSandLayer TRUE)))
(gimp-image-add-layer newImage middleSandLayer -1)
(set! topSandLayer (car (gimp-layer-copy bottomSandLayer TRUE)))
(gimp-image-add-layer newImage topSandLayer -1)
(gimp-image-set-active-layer newImage bottomSandLayer)
(set! bottomSandLayerCopy (car (gimp-layer-copy bottomSandLayer TRUE)))
(gimp-image-add-layer newImage bottomSandLayerCopy -1)
(gimp-layer-resize bottomSandLayerCopy (* 2 width) height width 0)
(gimp-drawable-offset bottomSandLayerCopy TRUE 0 (- width) 0)
(gimp-image-merge-down newImage bottomSandLayerCopy EXPAND-AS-NECESSARY)
(set! bottomSandLayer (car (gimp-image-get-active-layer newImage)))

(if (= emboss TRUE)
	(begin
	(plug-in-gauss-iir2 RUN-NONINTERACTIVE newImage bottomSandLayer 20.0 20.0)
	(plug-in-emboss RUN-NONINTERACTIVE newImage bottomSandLayer 30.0 45.0 20 TRUE)
	)
)


(gimp-image-set-active-layer newImage middleSandLayer)
(gimp-drawable-set-name middleSandLayer "Middle Sand Layer")
(plug-in-solid-noise runMode newImage middleSandLayer 1 1 (rand) 1 2.0 2.0)
(set! middleSandLayerCopy (car (gimp-layer-copy middleSandLayer TRUE)))
(gimp-image-add-layer newImage middleSandLayerCopy -1)
(gimp-layer-resize middleSandLayerCopy width (* 2 height) 0 0)
(gimp-drawable-offset middleSandLayerCopy TRUE 0 0 (- height))
(gimp-image-merge-down newImage middleSandLayerCopy EXPAND-AS-NECESSARY)
(set! middleSandLayer (car (gimp-image-get-active-layer newImage)))

(if (= emboss TRUE)
	(begin
	(plug-in-gauss-iir2 RUN-NONINTERACTIVE newImage middleSandLayer 20.0 20.0)
	(plug-in-emboss RUN-NONINTERACTIVE newImage middleSandLayer 30.0 45.0 20 TRUE)
	)
)

(gimp-image-set-active-layer newImage topSandLayer)
(gimp-drawable-set-name topSandLayer "Top Sand Layer")
(plug-in-solid-noise runMode newImage topSandLayer 1 1 (rand) 1 2.0 2.0)
(set! topSandLayerCopy (car (gimp-layer-copy topSandLayer TRUE)))
(gimp-image-add-layer newImage topSandLayerCopy -1)
(gimp-layer-resize topSandLayerCopy width (* 2 height) 0 height)
(gimp-drawable-offset topSandLayerCopy TRUE 0 0 height)
(gimp-image-merge-down newImage topSandLayerCopy EXPAND-AS-NECESSARY)
(set! topSandLayer (car (gimp-image-get-active-layer newImage)))

(if (= emboss TRUE)
	(begin
	(plug-in-gauss-iir2 RUN-NONINTERACTIVE newImage topSandLayer 20.0 20.0)
	(plug-in-emboss RUN-NONINTERACTIVE newImage topSandLayer 30.0 45.0 20 TRUE)
	)
)

(set! step (* -1 (/ height counter)))
(set! stepBottom (/ width counter))

(while (> counter 0)

(set! bottomSandLayerCopy (car (gimp-layer-copy bottomSandLayer TRUE)))
(gimp-image-add-layer newImage bottomSandLayerCopy -1)
(set! layerName (string-append "Frame " (number->string frameNum) " (" (number->string inSpeed) "ms)" " (replace)"))
(gimp-drawable-set-name bottomSandLayerCopy layerName)
(gimp-drawable-offset bottomSandLayerCopy TRUE OFFSET-BACKGROUND offsetBottom 0.0)
(set! middleSandLayerCopy (car (gimp-layer-copy middleSandLayer TRUE)))
(gimp-image-add-layer newImage middleSandLayerCopy -1)
(gimp-drawable-offset middleSandLayerCopy TRUE OFFSET-BACKGROUND 0.0 (* -1 offset))
(gimp-layer-set-mode middleSandLayerCopy middleBlendMode)
(gimp-image-merge-down newImage middleSandLayerCopy CLIP-TO-IMAGE)
(set! topSandLayerCopy (car (gimp-layer-copy topSandLayer TRUE)))
(gimp-image-add-layer newImage topSandLayerCopy -1)
(gimp-drawable-offset topSandLayerCopy TRUE OFFSET-BACKGROUND 0.0 offset)
(gimp-layer-set-mode topSandLayerCopy upperBlendMode)
(gimp-image-merge-down newImage topSandLayerCopy CLIP-TO-IMAGE)
(set! colorLayer (car (gimp-layer-copy bottomSandLayer TRUE)))
(gimp-image-add-layer newImage colorLayer -1)
(gimp-context-set-foreground color)
(gimp-edit-fill colorLayer FOREGROUND-FILL)
(gimp-layer-set-mode colorLayer colorBlend)
(gimp-image-merge-down newImage colorLayer CLIP-TO-IMAGE)


(set! offset (- offset step))
(set! offsetBottom (+ offsetBottom stepBottom))
(set! frameNum (+ frameNum 1))
(set! counter (- counter 1))

) ; goes with while

; Removes the original layer from the stack because it's no longer needed  
  
(gimp-image-remove-layer newImage bottomSandLayer) 
(gimp-image-remove-layer newImage middleSandLayer) 
(gimp-image-remove-layer newImage topSandLayer) 

(gimp-image-crop newImage inWidth inHeight 5 0)
(gimp-image-undo-enable newImage)


(gimp-context-pop)

(gimp-displays-flush)

; Displays the final animation

(gimp-display-new newImage)

  )
)

(script-fu-register "script-fu-flowing-sand-anim"
  "<Image>/Script-Fu/Fencepost/Animation/Animators/Flowing Sand..."
  "Create an animated lava effect"
  "Art Wade"
  "Art Wade"
  "2008"
  "RGB*"
  SF-IMAGE       	"Image" 0
  SF-DRAWABLE    	"Drawable" 0
  SF-ADJUSTMENT	"Image Width" '(200 100 2000 1 10 0 1)
  SF-ADJUSTMENT	"Image Height" '(200 100 2000 1 10 0 1)
  SF-ADJUSTMENT 	"Number of frames" '(30 15 2000 1 10 0 1)
  SF-ADJUSTMENT 	"Animation Speed (in ms)" '(150 10 500 1 1 0 1)
  SF-OPTION		"Upper Sand Layer Blend Mode" 		'("HARDLIGHT"
							"DIVIDE"
							"SCREEN"
							"OVERLAY"
							"DODGE"
							"BURN"
							"MULTIPLY"
							"SOFTLIGHT"
							"GRAIN EXTRACT"
							"GRAIN MERGE"
							"DIFFERENCE"
							"ADDITION"
							"SUBTRACT"
							"DARKEN ONLY"
							"LIGHTEN ONLY")
  SF-OPTION		"Middle Sand Layer Blend Mode" 		'("SOFTLIGHT"
							"DIVIDE"
							"SCREEN"
							"OVERLAY"
							"DODGE"
							"BURN"
							"HARDLIGHT"
							"MULTIPLY"
							"GRAIN EXTRACT"
							"GRAIN MERGE"
							"DIFFERENCE"
							"ADDITION"
							"SUBTRACT"
							"DARKEN ONLY"
							"LIGHTEN ONLY")
SF-TOGGLE		"Add Noise Interactively?"	FALSE
SF-TOGGLE		"Add Emboss Effect? (Grainy appearance)" TRUE
SF-COLOR		"Sand Color" 	'(188 154 100)
SF-OPTION		"Blend Mode of Color Layer" 		'("OVERLAY"
							"DIVIDE"
							"SCREEN"
							"DODGE"
							"BURN"
							"MULTIPLY"
							"HARDLIGHT"
							"SOFTLIGHT"
							"GRAIN EXTRACT"
							"GRAIN MERGE"
							"DIFFERENCE"
							"ADDITION"
							"SUBTRACT"
							"DARKEN ONLY"
							"LIGHTEN ONLY"
							"COLOR")
)


                         

