; GC Lens-Flare rel 0.01
; Created by Graechan
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
;
; ------------
;| Change Log |
; ------------ 
; Rel 0.01 - Initial Release
;
;;;; define new brush for drawing operation
(define (build-brush-circle brushName hardness spacing radius brush-color)

    (gimp-brush-new brushName)
	(gimp-brush-set-shape brushName BRUSH-GENERATED-CIRCLE)    
    (gimp-brush-set-spikes brushName 2)
    (gimp-brush-set-hardness brushName hardness)                   
    (gimp-brush-set-aspect-ratio brushName 1.0)
    (gimp-brush-set-angle brushName 0.0)                       
    (gimp-brush-set-spacing brushName spacing)
    (gimp-brush-set-radius brushName radius)            
    (gimp-context-set-brush brushName)
	(gimp-context-set-brush-default-size)
	(gimp-context-set-foreground brush-color))

;;Rem (gimp-brush-delete brushName)) ;delete the created brush
;
; Gradients blend direction list
(define list-blend-dir '("Left to Right" "Top to Bottom" "Diagonal to centre" "Diagonal from centre"))
;
; Include layer Procedure
(define (include-layer image newlayer oldlayer stack)	;stack 0=above 1=below
	(cond ((defined? 'gimp-image-get-item-position) ;test for 2.8 compatability
            (gimp-image-insert-layer image newlayer (car (gimp-item-get-parent oldlayer)) 
			(+ (car (gimp-image-get-item-position image oldlayer)) stack))                                     ;For GIMP 2.8 
          )
          (else
           (gimp-image-add-layer image newlayer (+ (car (gimp-image-get-layer-position image oldlayer)) stack)) ;For GIMP 2.6 
          )
    ) ;end cond
) ;end include layer procedure
;

;
(define (script-fu-gc-lens-flare image drawable
                               flare-color
							   flash-brushsize
							   flare-brushsize
							   flare-taper
							   leftx-adj
							   rightx-adj
							   smooth
							   glow-size
							   nova-opacity
							   path-info)

    (gimp-image-undo-group-start image)
	(if (and (= (car (gimp-drawable-mask-bounds drawable)) TRUE) (= (car (gimp-image-get-active-vectors image)) -1)) (begin
 (let* (
        (x1 (cadr (gimp-drawable-mask-bounds drawable)))                     ;x coordinate of the upper left corner of selection bounds.
        (y1 (caddr (gimp-drawable-mask-bounds drawable)))                    ;y coordinate of the upper left corner of selection bounds.
        (x2 (cadddr (gimp-drawable-mask-bounds drawable)))                   ;x coordinate of the lower right corner of selection bounds.
	    (y2 (cadr (cdddr (gimp-drawable-mask-bounds drawable))))             ;y coordinate of the lower right corner of selection bounds.
		(xc (+ (/ (- x2 x1) 2) x1))                                          ;x coordinate of the center of selection bounds.
		(yc (+ (/ (- y2 y1) 2) y1))                                          ;y coordinate of the center of selection bounds.
		(*newpoint* (cons-array 18 'double))
		(vectors 0)
	   )
	
	(aset *newpoint* 0 x1) ;x1 yc
	(aset *newpoint* 1 yc)
	(aset *newpoint* 2 x1)
	(aset *newpoint* 3 yc)
	(aset *newpoint* 4 x1)
	(aset *newpoint* 5 yc)
	(aset *newpoint* 6 xc) ;xc yc
	(aset *newpoint* 7 yc)
	(aset *newpoint* 8 xc)
	(aset *newpoint* 9 yc)
	(aset *newpoint* 10 xc)
	(aset *newpoint* 11 yc)
	(aset *newpoint* 12 x2) ;x2 yc
	(aset *newpoint* 13 yc)
	(aset *newpoint* 14 x2)
	(aset *newpoint* 15 yc)
	(aset *newpoint* 16 x2)
	(aset *newpoint* 17 yc)
	
	(set! vectors (car(gimp-vectors-new image "From Selection")))
	;(gimp-image-insert-vectors image vectors 0 0)
	(gimp-image-add-vectors image vectors 0)
	(gimp-vectors-stroke-new-from-points vectors 0 18 *newpoint* FALSE)
 )
   )) ;endif
	
	(let ((handler (car (gimp-message-get-handler))))
	(gimp-message-set-handler 0) ;{ MESSAGE-BOX (0), CONSOLE (1), ERROR-CONSOLE (2) }
	(cond ((= (car (gimp-image-get-active-vectors image)) -1) (error "No active path Found"))
	(else
							   
							  

 (let* (
            (width (car (gimp-drawable-width drawable)))   ;the width of the drawable
			(height (car (gimp-drawable-height drawable))) ;the height of the drawable
			(offx (car (gimp-drawable-offsets drawable)))  ;x offset of the drawable
			(offy (cadr (gimp-drawable-offsets drawable)))  ;y offset of the drawable
			(bkg-layer (car (gimp-layer-new image width height RGBA-IMAGE "bkg-layer" 100 NORMAL-MODE)))       ;create the bkg-layer
			(cFlare-layer (car (gimp-layer-new image width height RGBA-IMAGE "cFlare-layer" 100 NORMAL-MODE))) ;create the cFlare-layer
			(hFlare-layer (car (gimp-layer-new image width height RGBA-IMAGE "hFlare-layer" 100 NORMAL-MODE))) ;create the hFlare-layer
			(snova-layer (car (gimp-layer-new image width height RGBA-IMAGE "Supa Nova" nova-opacity SCREEN-MODE)))     ;create the snova-layer
			(lens-flare 0)
			(alpha (car (gimp-drawable-has-alpha drawable))) ;check if drawable has Alpha channel
		    (sel (car (gimp-selection-is-empty image))) ;check if selection exists
		    (layer-name (cond ((defined? 'gimp-image-get-item-position) (car (gimp-item-get-name drawable))) ;save the drawable name 2.8
		            (else (car (gimp-drawable-get-name drawable)))))                                         ;save the drawable name 2.6			
		    (orig-selection 0) ;save any existing selection 
			(ver 2.8) ;use to access the gimp version
			(inVectors (car (gimp-image-get-active-vectors image))) ;find the active path
			(num-strokes (car (gimp-vectors-get-strokes inVectors))) ;get the number of strokes
			(stroke-list (vector->list (cadr (gimp-vectors-get-strokes inVectors))))
			(stroke-id (car stroke-list)) ;get the stroke id
			(num-points (cadr (gimp-vectors-stroke-get-points inVectors stroke-id)))
			(control-points (vector->list (caddr (gimp-vectors-stroke-get-points inVectors stroke-id))))
			(anchor-point1x 0)
			(anchor-point1y 0)
			(anchor-point2x 0)
			(anchor-point2y 0)
			(anchor-point3x 0)
			(anchor-point3y 0)
			(leftx 0)
			(rightx 0)
			(leftx-adj (+ 100 leftx-adj))
			(left-dist 0)
			(leftx-pos 0)
			(rightx-adj (+ 100 rightx-adj))
			(right-dist 0)
			(rightx-pos 0)
			(co-ordCounter 0) ;prepare the co-ordinate counter for anchor-point routines
			
        )
	(cond ((not (defined? 'gimp-image-get-item-position)) (set! ver 2.6))) ;define the gimp version
	(if (not (= num-points 18)) (error "Wrong number of anchor points found\nAnchor points should = 3"))
	
	(set! co-ordCounter 2)
	(while (> co-ordCounter 0)
	(set! anchor-point1x (car control-points))
	(set! control-points (cdr control-points))
	(set! anchor-point1y (car control-points))
	(set! control-points (cdr control-points))
	(set! co-ordCounter (- co-ordCounter 1)))
	(set! control-points  (vector->list (caddr (gimp-vectors-stroke-get-points inVectors stroke-id))))
	
	
	(set! co-ordCounter 5)
	(while (> co-ordCounter 0)
	(set! anchor-point2x (car control-points))
	(set! control-points (cdr control-points))
	(set! anchor-point2y (car control-points))
	(set! control-points (cdr control-points))
	(set! co-ordCounter (- co-ordCounter 1)))
	(set! control-points  (vector->list (caddr (gimp-vectors-stroke-get-points inVectors stroke-id))))
	
	(set! co-ordCounter 8)
	(while (> co-ordCounter 0)
	(set! anchor-point3x (car control-points))
	(set! control-points (cdr control-points))
	(set! anchor-point3y (car control-points))
	(set! control-points (cdr control-points))
	(set! co-ordCounter (- co-ordCounter 1)))
	(set! control-points  (vector->list (caddr (gimp-vectors-stroke-get-points inVectors stroke-id))))
	
	(set! leftx (min anchor-point1x anchor-point3x))
	(set! rightx (max anchor-point1x anchor-point3x))
	
	(set! left-dist (- anchor-point2x leftx))
	(set! leftx-pos (- anchor-point2x (* (/ left-dist 100) leftx-adj)))
	(set! right-dist (- rightx anchor-point2x))
	(set! rightx-pos (+ anchor-point2x (* (/ right-dist 100) rightx-adj)))
	
	;(gimp-image-undo-group-start image)
	(gimp-context-push)
    (gimp-context-set-paint-method "gimp-paintbrush")
	(cond ((defined? 'gimp-context-set-dynamics) (gimp-context-set-dynamics "Dynamics Off")))
    (gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))

	(if (= alpha FALSE) (gimp-layer-add-alpha drawable))
	
;;;;check that a selection was made if so delete it	
	(if (= sel FALSE) (begin
	(set! orig-selection (car (gimp-selection-save image))) ;save any existing selection
	(gimp-selection-none image)
	)) ;endif for existing selection

;;;;begin the script
    (include-layer image bkg-layer drawable 0)	;[stack 0=above 1=below] add the bkg layer to the image
	(gimp-layer-set-offsets bkg-layer offx offy)
	(gimp-drawable-fill bkg-layer 0) ;fill the bkg layer with black
	(include-layer image cFlare-layer bkg-layer 0)	;[stack 0=above 1=below] add the cFlare layer to the image 
	(gimp-layer-set-offsets cFlare-layer offx offy)
	(include-layer image hFlare-layer cFlare-layer 0)	;[stack 0=above 1=below] add the hFlare layer to the image
	(gimp-layer-set-offsets hFlare-layer offx offy)
;;-------------------------------------------------------------------------------------------------------------------create center flash	
	(let ((*newpoint* (cons-array 2 'double)))
	(aset *newpoint* 0 anchor-point2x)   ; set the paint array point
	(aset *newpoint* 1 anchor-point2y)
	(build-brush-circle "soft circle" .5 1 (/ flash-brushsize 2) '(255 255 255))
	(gimp-paintbrush-default cFlare-layer 2 *newpoint*)) ;draw the center segment of the flare
	(gimp-brush-delete "soft circle") ;delete the created brush
;;---------------------------------------------------------------------------------------------------------------------create left flare			
	(let ((*newpoint* (cons-array 4 'double)))
	(aset *newpoint* 0 anchor-point2x)   ; set the paint array point ; set the paint array points
	(aset *newpoint* 1 anchor-point2y)
	(aset *newpoint* 2 leftx-pos)
	(aset *newpoint* 3 anchor-point2y)
	(build-brush-circle "soft circle" .5 1 (/ flare-brushsize 5) '(255 255 255))
	(gimp-paintbrush-default hFlare-layer 4 *newpoint*)) ;draw the horizontal segment of the flare	
	(gimp-brush-delete "soft circle") ;delete the created brush
;;-------------------------------------------------------------------------------------------------------create right flare	
	(let ((*newpoint* (cons-array 4 'double)))
	(aset *newpoint* 0 anchor-point2x)   ; set the paint array point ; set the paint array points
	(aset *newpoint* 1 anchor-point2y)
	(aset *newpoint* 2 rightx-pos)
	(aset *newpoint* 3 anchor-point2y)
	(build-brush-circle "soft circle" .5 1 (/ flare-brushsize 5) '(255 255 255))
	(gimp-paintbrush-default hFlare-layer 4 *newpoint*)) ;draw the horizontal segment of the flare	
	(gimp-brush-delete "soft circle") ;delete the created brush
;;--------------------------------------------------------------------------------------------------------------	

	(plug-in-gauss-rle2 RUN-NONINTERACTIVE image hFlare-layer (* flare-taper 2) 20)
	
	(let ((copy-layer 0))
	(set! copy-layer (car (gimp-layer-copy cFlare-layer TRUE)))
	(include-layer image copy-layer cFlare-layer 0)	;[stack 0=above 1=below] add the bkg layer to the image
	(plug-in-gauss-rle2 RUN-NONINTERACTIVE image copy-layer 150 0)
	(set! cFlare-layer (car (gimp-image-merge-down image copy-layer EXPAND-AS-NECESSARY)))
	) ;endlet
	
	(let ((copy-layer 0))
	(set! copy-layer (car (gimp-layer-copy hFlare-layer TRUE)))
	(include-layer image copy-layer hFlare-layer 0)	;[stack 0=above 1=below] add the bkg layer to the image
	(plug-in-gauss-rle2 RUN-NONINTERACTIVE image copy-layer 200 200)
	(set! hFlare-layer (car (gimp-image-merge-down image copy-layer EXPAND-AS-NECESSARY)))
	) ;endlet
	
	(set! lens-flare (car (gimp-image-merge-down image hFlare-layer EXPAND-AS-NECESSARY))) ;--------------create the lens flare
	
	(let ((copy-layer 0))
	(set! copy-layer (car (gimp-layer-copy lens-flare TRUE)))
	(include-layer image copy-layer lens-flare 0)	;[stack 0=above 1=below] add the bkg layer to the image
	(gimp-context-set-interpolation INTERPOLATION-LANCZOS) ;-------------set the interpolation
	(gimp-layer-scale  copy-layer width (/ height 2) FALSE) ;------------scale the copy-layer
	(gimp-layer-set-offsets copy-layer offx (+ offy (- anchor-point2y (/ anchor-point2y 2)))) ;0ffset the copy-layer
	(set! lens-flare (car (gimp-image-merge-down image copy-layer EXPAND-AS-NECESSARY))) ;merge the lens-flare layers 
	) ;endlet
	
    (set! lens-flare (car (gimp-image-merge-down image lens-flare EXPAND-AS-NECESSARY)))
;	(plug-in-colorify RUN-NONINTERACTIVE image lens-flare flare-color) ;--------------add color to the lens-flare
	(gimp-color-balance lens-flare 0 TRUE 40 10 -15) ;{ SHADOWS (0), MIDTONES (1), HIGHLIGHTS (2) }
    (gimp-color-balance lens-flare 1 TRUE 30 0 -35)
    (gimp-color-balance lens-flare 2 TRUE 20 0 -40)
	(gimp-hue-saturation lens-flare 0 0 0 40)
	(if (> smooth 0) (plug-in-gauss-rle2 RUN-NONINTERACTIVE image lens-flare smooth smooth))
	
	(include-layer image snova-layer lens-flare 0)	;[stack 0=above 1=below] add the snova-layer to the image
	(gimp-context-set-foreground '(0 0 0))
	(gimp-drawable-fill snova-layer FOREGROUND-FILL)
	(plug-in-nova 1 image 
                       snova-layer ;drawable 
					   anchor-point2x ;xcenter 
					   anchor-point2y ;ycenter 
					   flare-color ;color 
					   glow-size ;radius 
					   32 ;nspoke 
					   0) ;randomhue
	
	
	
	
	(set! hFlare-layer (car (gimp-image-merge-down image (car (gimp-image-get-active-layer image)) EXPAND-AS-NECESSARY)))
	(plug-in-colorify RUN-NONINTERACTIVE image hFlare-layer flare-color)	
	
	(let ((copy-layer 0))
	(set! copy-layer (car (gimp-layer-copy hFlare-layer TRUE)))
	(include-layer image copy-layer hFlare-layer 0)	;[stack 0=above 1=below] add the bkg layer to the image
	(set! hFlare-layer (car (gimp-image-merge-down image copy-layer EXPAND-AS-NECESSARY))))
	(gimp-layer-set-mode hFlare-layer SCREEN-MODE)	
	(cond ((= ver 2.8) (gimp-item-set-name hFlare-layer "Lens Flare"))
	(else (gimp-drawable-set-name hFlare-layer "Lens Flare"))
    ) ;endcond
	
;;;;display to console info about settings
    (if (= path-info TRUE) (begin
	(gimp-message (string-append "num-strokes" "_" (number->string num-strokes) "\nstroke-id" "_" 
	(number->string stroke-id) "\nnum-points" "_" (number->string num-points) "\ncontrol-points" "\n" 
	(unbreakupstr (map number->string control-points) " ") "\n" 
	"\nCenter Flash Point" ":" (number->string anchor-point2x) "," (number->string anchor-point2y) 
	"\nLeft Flare Point" ":" (number->string leftx-pos) "," (number->string anchor-point2y) 
	"\nRight Flare Point" ":" (number->string rightx-pos) ","	(number->string anchor-point2y)
    )) ;endmessage	
	)) ;endif(path-info)
	(if (= sel FALSE) (gimp-selection-load orig-selection))

;;;;finish the script	
	
	(if (= sel FALSE) (begin
	(gimp-selection-load orig-selection)
	(gimp-image-remove-channel image orig-selection)
	)) ;endif 
	(gimp-image-remove-vectors image inVectors)


	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
	(gimp-context-pop)

 ) ;end Variables    
	 
 )) ;end ConditionalStart
 (gimp-message-set-handler handler))
) ;end Procedure

(script-fu-register "script-fu-gc-lens-flare"        		    
  "GC Lens-Flare..."
  "Creates an ajustable Lens Fare from a selection or a path with 3 anchors"
  "Graechan"
  "Graechan - http://gimpchat.com"
  "Dec 2014"
  "RGB*"
  SF-IMAGE      "image"      0
  SF-DRAWABLE   "drawable"   0
  SF-COLOR      "Flare color"  '(255 179 89)
  SF-ADJUSTMENT "Flash Size" '(50 0 200 1 10 0 0)
  SF-ADJUSTMENT "Flare Size" '(20 0 200 1 10 0 0)
  SF-ADJUSTMENT "Flare Taper Length" '(100 0 500 1 10 0 0)
  SF-ADJUSTMENT "Left Flare Adj (percent)" '(0 -100 100 1 10 0 0)
  SF-ADJUSTMENT "Right Flare Adj (percent)" '(0 -100 100 1 10 0 0)
  SF-ADJUSTMENT "Flare Smooth" '(5 0 10 1 1 0 0)
  SF-ADJUSTMENT "Glow Size" '(10 0 200 1 10 0 0)
  SF-ADJUSTMENT "Glow opacity" '(50 0 100 1 10 0 0)
  SF-TOGGLE     "Display Path Information"   FALSE
)

(script-fu-menu-register "script-fu-gc-lens-flare" "<Image>/Filters/Light and Shadow")


