; Font Display rel 0.05
; Created by Graechan
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
;
; ------------
;| Change Log |
; ------------ 
; Rel 0.01 - Initial Release
; Rel 0.02 - Changed Layout-1 to one posted by Barbie, thankyou Barbie, Also added a Pattern background option as suggested by Anmal
; Rel 0.03 - Added a letter spacing adjustment
; Rel 0.04 - Changed the header text to always use the 'Sans' font   
; Rel 0.05 - Changed the header text to optionally use the 'Sans' font
;
(define (script-fu-font-display 
                                      your-text
									  color 
                                      font-in-out
									  active
									  header
									  display-layout
									  font-size
									  letter-spacing
									  bkg-type
                                      bkg-color
									  bkg-pattern
									  )
  (let* (
         (image (car (gimp-image-new 50 50 RGB)))
         
         (border (/ font-size 4))
		 (font-in (if (= active TRUE) (car (gimp-context-get-font)) font-in-out))
		 (font (if (> (string-length font-in) 0) font-in (car (gimp-context-get-font))))
		 (layout-1 (string-append  your-text "\n" " !\"#$%&'()*+,-.>\n:;<=?@/[ \\ ]^_`\n0123456789\nABCDEFGHIJKLM\nNOPQRSTUVWXYZ\nabcdefghijklm\nnopqrstuvwxyz"))
		 (layout-2 (string-append  your-text "\n" " !\"#$%&'()*+,-./:;<=>?@[ \\ ]^_`\n0123456789\nABCDEFGHIJKLMNOPQRSTUVWXYZ\nabcdefghijklmnopqrstuvwxyz"))
		 (text (if (= display-layout 0) layout-1 layout-2))
         (header-layer 0)
		 (header-width 0)
		 (header-height 0)
		 (text-layer 0)
         (width 0)
         (height 0)		 
         (bkg-layer 0)
		 (header-font font)         
         )
		 
    (gimp-context-push)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))

;;;;create the layers	
	(if (= header TRUE) (set! header-font "Sans"))
	(set! header-layer (car (gimp-text-fontname image -1 0 0 font border TRUE font-size PIXELS header-font)))
	(gimp-text-layer-set-justification header-layer 2)
	(set! header-width (car (gimp-drawable-width header-layer)))
	(set! header-height (car (gimp-drawable-height header-layer)))
	(set! text-layer (car (gimp-text-fontname image -1 0 0 text border TRUE font-size PIXELS font)))
	(set! width (car (gimp-drawable-width text-layer)))
    (set! height (car (gimp-drawable-height text-layer)))
    		
;;;;position the text on image
	(gimp-text-layer-set-justification text-layer 2)
	(gimp-text-layer-set-letter-spacing text-layer letter-spacing)
	(set! width (car (gimp-drawable-width text-layer)))
    (set! height (car (gimp-drawable-height text-layer)))
	(gimp-layer-set-offsets text-layer 0 header-height)
	(gimp-layer-set-offsets header-layer (/ (- width header-width) 2) 0)	
	(set! text-layer (car (gimp-image-merge-down image text-layer EXPAND-AS-NECESSARY)))
	
;;;;set the text layer colors
    (gimp-image-resize-to-layers image)
    (gimp-context-set-foreground color)
	(gimp-selection-layer-alpha text-layer)
	(gimp-edit-fill text-layer FOREGROUND-FILL)
	(gimp-selection-invert image)
	(gimp-context-set-background bkg-color)
	(gimp-context-set-pattern bkg-pattern)
	(if (= bkg-type 1) (gimp-edit-fill text-layer BACKGROUND-FILL))
	(if (= bkg-type 2) (gimp-edit-fill text-layer PATTERN-FILL))

;;;;finish the image
	(gimp-selection-none image)
	(gimp-drawable-set-name text-layer font)
	(gimp-context-pop)
	
    (gimp-display-new image)
    )
  ) 
(script-fu-register "script-fu-font-display"
  "Font Display..."
  "Displays a font in your choice of color on a background color of your choice"
  "Graechan"
  "Graechan - http://gimpchat.com"
  "May 2013"
  ""
  SF-TEXT      "Your Text:"         "Your Text"
  SF-COLOR      "Font color"         '(0 0 0)
  SF-FONT       "Font to Display"               "JasmineUPC Bold"
  SF-TOGGLE     "Or Display the Active Font"   TRUE
  SF-TOGGLE     "Use 'Sans' for Header"   FALSE
  SF-OPTION "Display Layout" '("Layout-1" "Layout-2")
  SF-ADJUSTMENT "Font size (pixels)" '(40 6 800 1 10 0 1)
  SF-ADJUSTMENT "Letter Spacing" '(0 -100 100 1 10 0 0)
  SF-OPTION "Background Type" '("Transparent" "Color" "Pattern")
  SF-COLOR      "Background color"         '(153 153 153)
  SF-PATTERN    "Pattern"            "Pink Marble"  
  )
(script-fu-menu-register "script-fu-font-display" "<Fonts>/")